# BTW

This project is a NumPy implementation of [Sampling Subgraphs with Guaranteed
Treewidth for Accurate and Efficient Graphical Inference](https://jaeminyoo.github.io/resources/2019/WSDM/YooWSDM19.pdf) ([WSDM 2020](http://www.wsdm-conference.org/2020/)).
This paper proposes a novel algorithm for sampling subgraphs of a large graph for accurate and efficient graphical inference.

<p align="center">
    <img src="docs/overview.png" alt="" width="860">
</p>

## Prerequisites

- Python 3.5+
- [NumPy](https://numpy.org)
- [scikit-learn](https://scikit-learn.org/)

## Usage

You can run the demo script by `bash demo.sh`, which simply moves to `src` and
runs `python main.py`.
The script samples a small subgraph by our BTW sampling, stores it at `out`, and
runs both accurate and exact inference on it.
The running time and accuracy are different in the two inference algorithms
because of their characteristics. 
You can change the dataset and function parameters by modifying `src/main.py`.
The core functions are implemented in `src/btw`.

The following figure shows the effectiveness of our BTW sampling.
First, BTW enables the tractable inference of the junction tree algorithm on
generated subgraphs, because it is guaranteed to generate subgraphs of bounded
treewidth.
Second, BTW speeds up loopy belief propagation without hurting its accuracy,
because it samples important and essential edges of the original graph.  

<p align="center">
    <img src="docs/performance.png" alt="" width="480">
</p>

## Reference

You can download [this bib file](docs/YooKSCZ20.bib) or copy the following information: 

```
@inproceedings{YooKSCZ20,
  author    = {Jaemin Yoo and
               U Kang and
               Mauro Scanagatta and
               Giorgio Corani and
               Marco Zaffalon},
  title     = {Sampling Subgraphs with Guaranteed Treewidth for Accurate and Efficient Graphical Inference},
  booktitle = {ACM International Conference on Web Search and Data Mining (WSDM)},
  year      = {2020}
}
```
